SystemOrganization addCategory: #'Announcements-Core'!

Set subclass: #AnnouncementSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!AnnouncementSet methodsFor: 'adding' stamp: 'lr 6/13/2006 08:13'!
, anAnnouncementClass
	self add: anAnnouncementClass! !

!AnnouncementSet methodsFor: 'testing' stamp: 'lr 10/3/2006 14:31'!
handles: anAnnouncementClass
	^ self anySatisfy: [ :each | each handles: anAnnouncementClass ]! !

Object subclass: #Announcement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!Announcement commentStamp: 'Tbn 11/12/2010 10:57' prior: 0!
This class is the superclass for events that someone might want to announce, such as a button click or an attribute change. Typically you create subclasses for your own events you want to announce.

 
!

!Announcement class methodsFor: 'public' stamp: 'lr 9/20/2006 08:18'!
, anAnnouncementClass
	^ AnnouncementSet with: self with: anAnnouncementClass! !

!Announcement class methodsFor: 'converting' stamp: 'lr 10/3/2006 14:31'!
asAnnouncement
	^ self new! !

!Announcement class methodsFor: 'testing' stamp: 'lr 10/3/2006 14:31'!
handles: anAnnouncementClass
	^ anAnnouncementClass isKindOf: self! !

!Announcement methodsFor: 'converting' stamp: 'lr 10/3/2006 14:32'!
asAnnouncement
	^ self! !

Object subclass: #Announcer
	instanceVariableNames: 'subscriptions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!Announcer commentStamp: 'lr 3/2/2009 10:27' prior: 0!
The code is based on the announcements as described by Vassili Bykov in <http://www.cincomsmalltalk.com/userblogs/vbykov/blogView?searchCategory=Announcements%20Framework>. The implementation is a slightly extended and generalized version of the code found in OmniBrowser by Colin Putney.!

!Announcer methodsFor: 'private' stamp: 'lr 2/26/2011 11:26'!
actionSequence
	^ ActionSequence new! !

!Announcer methodsFor: 'announce' stamp: 'lr 8/5/2008 12:06'!
announce: anAnnouncement
	| announcement |
	announcement := anAnnouncement asAnnouncement.
	subscriptions ifNil: [ ^ announcement ].
	subscriptions keysAndValuesDo: [ :class :actions |
		(class handles: announcement) 
			ifTrue: [ actions valueWithArguments: (Array with: announcement) ] ].
	^ announcement! !

!Announcer methodsFor: 'private' stamp: 'lr 2/26/2011 11:31'!
messageSendClass
	^ MessageSend! !

!Announcer methodsFor: 'private' stamp: 'lr 2/26/2011 11:29'!
normalize: anActionSequence
	^ anActionSequence! !

!Announcer methodsFor: 'convenience' stamp: 'lr 10/27/2006 14:26'!
on: anAnnouncementClass do: aValuable
	^ self subscribe: anAnnouncementClass do: aValuable! !

!Announcer methodsFor: 'convenience' stamp: 'lr 10/27/2006 14:27'!
on: anAnnouncementClass send: aSelector to: anObject
	^ self subscribe: anAnnouncementClass send: aSelector to: anObject! !

!Announcer methodsFor: 'subscription' stamp: 'lr 2/26/2011 11:31'!
subscribe: anAnnouncementClass do: aValuable 
	| actions |
	subscriptions ifNil: [ subscriptions := IdentityDictionary new ].
	actions := subscriptions at: anAnnouncementClass ifAbsent: [ self actionSequence ].
	subscriptions at: anAnnouncementClass put: (self normalize: (actions copyWith: aValuable)).
	^ aValuable! !

!Announcer methodsFor: 'subscription' stamp: 'lr 2/26/2011 08:31'!
subscribe: anAnnouncementClass send: aSelector to: anObject
	^ self subscribe: anAnnouncementClass do: (self messageSendClass receiver: anObject selector: aSelector)! !

!Announcer methodsFor: 'subscription' stamp: 'lr 2/26/2011 11:28'!
unsubscribe: anObject
	subscriptions ifNil: [ ^ self ].
	subscriptions keysAndValuesDo: [ :class :actions |
		subscriptions at: class put: (self normalize: (actions
			reject: [ :each | each receiver == anObject ])) ].
	subscriptions keysAndValuesRemove: [ :class :actions |
		actions isEmpty ]! !

!Announcer methodsFor: 'convenience' stamp: 'tg 2/25/2009 12:05'!
when: anAnnouncementClass do: aValuable
	^ self subscribe: anAnnouncementClass do: aValuable! !

Announcer subclass: #WeakAnnouncer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!WeakAnnouncer commentStamp: 'lr 2/26/2011 11:44' prior: 0!
An announcer that only holds weakly onto the recievers. Avoid the use of this class, instead consider redesigning your application so that it properly unregisters unused events. Don't be surprised if this class causes more and harder problems than it solves. This is caused by the weak design of the weak datastructures in Pharo. Note that blocks never disappear, they are not referenced weakly even when using this announcer.!

!WeakAnnouncer methodsFor: 'private' stamp: 'lr 2/26/2011 11:27'!
actionSequence
	^ WeakActionSequence new! !

!WeakAnnouncer methodsFor: 'private' stamp: 'lr 2/26/2011 11:32'!
messageSendClass
	^ WeakMessageSend! !

!WeakAnnouncer methodsFor: 'private' stamp: 'lr 2/26/2011 11:29'!
normalize: anActionSequence
	^ anActionSequence select: [ :each | each isValid ]! !
